"""
Utility module

"""

import numbers

def validate_timestamp(timestamp):
    return timestamp is not None

def validate_tags(tags):
    return isinstance(tags, dict)

def validate_prediction(prediction):
    return isinstance(prediction, str)

def validate_confidence(confidence):
    return isinstance(confidence, numbers.Number)

def validate_features(features):
    return isinstance(features, dict)

def validate_embeddings(embeddings):
    return isinstance(embeddings, list)

def validate_image_url(image_url):
    return isinstance(image_url, str)

def validate_groundtruth(groundtruth):
    return isinstance(groundtruth, str)

def add_prefix_to_keys(dictionary, prefix):
    return_dict = {}
    for key in dictionary:
        return_dict[prefix + '.' + key] = dictionary[key]

    return return_dict



import datetime
import json

from dioptra.client import Client
from dioptra.utils import (
    validate_tags,
    validate_embeddings,
    validate_features,
    validate_groundtruth,
    validate_prediction,
    validate_confidence,
    validate_image_url,
    validate_timestamp,
    add_prefix_to_keys
)

class Logger:
    """
    Dioptra logger client

    """

    def __init__(
        self,
        api_key,
        endpoint_url='https://api.dioptra.ai/demo'
    ):
        self.api_key = api_key
        self.endpoint_url = endpoint_url
        self.event_url = endpoint_url + '/events'
        self._headers = {
            'x-api-key': api_key,
            'Content-Type': 'application/json'
        }

    def log(
        self,
        request_id,
        timestamp=None,
        model_id=None,
        model_version=None,
        groundtruth=None,
        prediction=None,
        confidence=None,
        features=None,
        embeddings=None,
        image_url=None,
        tags=None
    ):

        payload = self.package_payload(
            model_id,
            model_version,
            timestamp,
            request_id,
            groundtruth,
            prediction,
            confidence,
            features,
            embeddings,
            image_url,
            tags
        )

        client = Client()
        my_response = client.call(
            payload=payload,
            endpoint_url=self.event_url,
            headers=self._headers)

        return my_response

    def package_payload(
        self,
        model_id,
        model_version,
        timestamp,
        request_id,
        groundtruth,
        prediction,
        confidence,
        features,
        embeddings,
        image_url,
        tags
    ):

        payload = {
            'request_id': request_id
        }

        if model_id:
            payload['model_id'] = model_id

        if model_version:
            payload['model_version'] = model_version

        if timestamp and validate_timestamp(timestamp):
            payload['timestamp'] = timestamp
        else:
            payload['timestamp'] = datetime.datetime.utcnow().isoformat()

        if groundtruth and validate_groundtruth(groundtruth):
            payload['groundtruth'] = groundtruth

        if prediction and validate_prediction(prediction):
            payload['prediction'] = prediction

        if confidence and validate_confidence(confidence):
            payload['confidence'] = confidence

        if features and validate_features(features):
            prefixed_features = add_prefix_to_keys(features, 'feature')
            payload.update(prefixed_features)

        if image_url and validate_image_url(image_url):
            payload['feature.image_url'] = image_url

        if embeddings and validate_embeddings(embeddings):
            payload['feature.embeddings'] = json.dumps(embeddings)

        if tags and validate_tags(tags):
            prefixed_tags = add_prefix_to_keys(tags, 'tag')
            payload.update(prefixed_tags)

        return [payload]



import logging
import requests
from requests.adapters import HTTPAdapter
from requests.packages.urllib3.util.retry import Retry

class Client:

    def __init__(self):

        retry_strategy = Retry(
            total=3,
            backoff_factor=1,
            status_forcelist=[429, 500, 502, 503, 504],
            method_whitelist=['POST']
        )

        adapter = HTTPAdapter(max_retries=retry_strategy)

        self.http_session = requests.Session()
        self.http_session.mount('https://', adapter)
        self.timeout=10

    def call(self, payload, endpoint_url, headers):

        response = self.http_session.post(
            timeout=self.timeout,
            url=endpoint_url,
            headers=headers,
            json=payload)

        if response.status_code != requests.codes.ok:
            logging.error('There was an error calling the API')
            response.raise_for_status()

        return response.json()

